/*
Copyright The KCP Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"context"
	"time"

	logicalcluster "github.com/kcp-dev/logicalcluster"

	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"

	v1alpha1 "github.com/kcp-dev/kcp/pkg/apis/tenancy/v1alpha1"
	scheme "github.com/kcp-dev/kcp/pkg/client/clientset/versioned/scheme"
)

// ClusterWorkspacesGetter has a method to return a ClusterWorkspaceInterface.
// A group's client should implement this interface.
type ClusterWorkspacesGetter interface {
	ClusterWorkspaces() ClusterWorkspaceInterface
}

// ClusterWorkspaceInterface has methods to work with ClusterWorkspace resources.
type ClusterWorkspaceInterface interface {
	Create(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.CreateOptions) (*v1alpha1.ClusterWorkspace, error)
	Update(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.UpdateOptions) (*v1alpha1.ClusterWorkspace, error)
	UpdateStatus(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.UpdateOptions) (*v1alpha1.ClusterWorkspace, error)
	Delete(ctx context.Context, name string, opts v1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error
	Get(ctx context.Context, name string, opts v1.GetOptions) (*v1alpha1.ClusterWorkspace, error)
	List(ctx context.Context, opts v1.ListOptions) (*v1alpha1.ClusterWorkspaceList, error)
	Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.ClusterWorkspace, err error)
	ClusterWorkspaceExpansion
}

// clusterWorkspaces implements ClusterWorkspaceInterface
type clusterWorkspaces struct {
	client  rest.Interface
	cluster logicalcluster.Name
}

// newClusterWorkspaces returns a ClusterWorkspaces
func newClusterWorkspaces(c *TenancyV1alpha1Client) *clusterWorkspaces {
	return &clusterWorkspaces{
		client:  c.RESTClient(),
		cluster: c.cluster,
	}
}

// Get takes name of the clusterWorkspace, and returns the corresponding clusterWorkspace object, and an error if there is any.
func (c *clusterWorkspaces) Get(ctx context.Context, name string, options v1.GetOptions) (result *v1alpha1.ClusterWorkspace, err error) {
	result = &v1alpha1.ClusterWorkspace{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of ClusterWorkspaces that match those selectors.
func (c *clusterWorkspaces) List(ctx context.Context, opts v1.ListOptions) (result *v1alpha1.ClusterWorkspaceList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.ClusterWorkspaceList{}
	err = c.client.Get().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested clusterWorkspaces.
func (c *clusterWorkspaces) Watch(ctx context.Context, opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a clusterWorkspace and creates it.  Returns the server's representation of the clusterWorkspace, and an error, if there is any.
func (c *clusterWorkspaces) Create(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.CreateOptions) (result *v1alpha1.ClusterWorkspace, err error) {
	result = &v1alpha1.ClusterWorkspace{}
	err = c.client.Post().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(clusterWorkspace).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a clusterWorkspace and updates it. Returns the server's representation of the clusterWorkspace, and an error, if there is any.
func (c *clusterWorkspaces) Update(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.UpdateOptions) (result *v1alpha1.ClusterWorkspace, err error) {
	result = &v1alpha1.ClusterWorkspace{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		Name(clusterWorkspace.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(clusterWorkspace).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *clusterWorkspaces) UpdateStatus(ctx context.Context, clusterWorkspace *v1alpha1.ClusterWorkspace, opts v1.UpdateOptions) (result *v1alpha1.ClusterWorkspace, err error) {
	result = &v1alpha1.ClusterWorkspace{}
	err = c.client.Put().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		Name(clusterWorkspace.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(clusterWorkspace).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the clusterWorkspace and deletes it. Returns an error if one occurs.
func (c *clusterWorkspaces) Delete(ctx context.Context, name string, opts v1.DeleteOptions) error {
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *clusterWorkspaces) DeleteCollection(ctx context.Context, opts v1.DeleteOptions, listOpts v1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched clusterWorkspace.
func (c *clusterWorkspaces) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts v1.PatchOptions, subresources ...string) (result *v1alpha1.ClusterWorkspace, err error) {
	result = &v1alpha1.ClusterWorkspace{}
	err = c.client.Patch(pt).
		Cluster(c.cluster).
		Resource("clusterworkspaces").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
